﻿########################
## LCA mortality data ##
########################

#-# Import the packages
#install.packages("poLCA")
#install.packages("fields")
library(poLCA)
library(fields)

#-#Create synt data
#this data have similar pattern frequencies as the cause of death-male data
n = 15000 
P = 30    ##number of variables

freq_patt = c(1.371282e-01, 2.712019e-01, 2.657867e-01, 1.723019e-01,
              9.560435e-02, 4.015315e-02, 1.305580e-02, 3.710285e-03,
              8.038245e-04, 2.115328e-04, 2.961459e-05, 1.269197e-05)
synt_data = matrix( 0, n, P)
for ( i in 1:n){
  v_p <- sample( 1:P, sample(1:12, 1,prob = freq_patt))
  synt_data[i, v_p ] <- 1
}


#######
# LCA #
#######

data <- synt_data
data2 <- as.matrix(data) +1
colnames(data2) <- sapply(1:P, function(x) paste( "V-", x, sep = ""))
n = nrow(data) #number of observations
P = ncol(data) #number of death-causes
C = 2 # number of latent classes
mod <- poLCA(data2~ 1, maxiter= 1, nclass= C, nrep= 1, data= as.data.frame(data2)) 

PI = mod$probs  #ex conditional probs
tau = mod$P #ex weights
theta = colSums(data)/sum(data)

############################
# Model selection criteria #
############################
print("")
print("Model selection criteria")

BIC= mod$bic
z = mod$posterior
C1 <- matrix(0, n, ncol(z))
for (i in 1:n){
  C1[i, which.max(z[i, ])] <- 1
  ICL <- BIC + 2 * sum(C1 * ifelse(z > 0, log(z), 0))
}

print(paste( "BIC = ", round(BIC, 3)))
print(paste( "ICL = ", round(ICL, 3)))

##################
# Entropy tables #
##################

#-# Entropy at individual level
entropy_r_i <- sapply(1:n, function(x) -sum(mod$posterior[x, ]*log(mod$posterior[x, ]), na.rm= T))/
  log(C) #individual relative entropy

entropy_r_i_C <- matrix(NA, ncol= C, nrow= n) #entropy distribution for each class
colnames(entropy_r_i_C) = sapply(1:C, function(x) paste("class-", x))
for ( i in 1:n){
  entropy_r_i_C[i, mod$predclass[i]]=  entropy_r_i[i]
}

View(entropy_r_i_C)
summary(entropy_r_i_C[,1], na.rm =T) # example: entropy distribution for the 1st class

#-# Entropy at pattern level
entropy_r_p_C <- rep(NA, C)
for (cc in 1:C){
  patt_c= data[which(mod$predclass== cc ), ]
  patt_c_2= apply(patt_c, 1, function(x) paste(as.character(x), collapse= ""))
  patt_c_3= as.numeric(table(patt_c_2))
  patt_c_4= patt_c_3/sum(patt_c_3)
  entropy_p = -sum(patt_c_4*log( patt_c_4))/(log(length(patt_c_4)))
  entropy_r_p_C[cc] <- entropy_p
}
print("")
print("Class entropy at pattern level:")
print(cbind(sapply(1:C, function(x) paste("class-", x)), entropy_r_p_C))

####################
# Diagnostic plots #
####################

pdf("LCA_plots.pdf")

#-# Death cause-class heatmaps
heatmapu_diff= heatmapu_cprobs= matrix(rep(NA, C*P), ncol= P)
colnames(heatmapu_diff)= colnames(heatmapu_cprobs)= sapply(colnames(data2), 
                                                           function(x) substr(x, 1, 13))
rownames(heatmapu_diff)= rownames(heatmapu_cprobs) = seq(1, C)

for ( cc  in 1 : C){
  temp1 <- sapply(PI, function(l) l[cc, ])
  temp2 <- temp1[2, ]/theta
  
  heatmapu_cprobs[cc, ] <- temp1[2, ]
  heatmapu_diff[cc, ] <- temp2
}

darkblue <- rgb(0,0,61, maxColorValue = 256)
perc <- paste(round(tau*100, 2), "%", sep="")
pal <- colorRampPalette(c("azure3", "dodgerblue4", darkblue))
par(mar= c(4, 5, 3.5, 4))

# Differentials heatmap
temp3= heatmapu_diff[, order(colnames(heatmapu_diff))]
image( 1:C, 1:P, temp3, col= c(rep("white", 1), pal(400)),axes= FALSE, xlab= "", ylab= "", 
       main= "Death cause - class, differentials heatmap")
abline(v= 0:C+ 0.5, col = "gray")
abline(h= 0:P+ 0.5, col = "gray")
mtext(rownames(temp3), side= 1, at= 1:C, las= 1, cex= 0.7, adj = 0.5)
mtext(perc, side= 3, at= 1:C, las= 1, cex= .7, adj= .4)
mtext(colnames(temp3), side= 2, at= 1:P, las= 1, cex= 0.7, adj= 1.03)

# Conditonal probabilities
temp4 = heatmapu_cprobs[, order(colnames(heatmapu_cprobs))]
image( 1:C, 1:P, temp4, col= c(rep("white", 1), pal(400)), axes= FALSE, xlab= "", yla = "", 
       main= "Death cause - class, conditional probabilities heatmap")
abline(v= 0:C+ 0.5, col= "gray")
abline(h= 0:P+ 0.5, col= "gray")
mtext(rownames(temp4), side= 1, at= 1:C, las = 1, cex = 0.7, adj = 0.5)
mtext(perc, side = 3, at= 1:C, las= 1, cex= .7, adj = .4)
mtext(colnames(temp4), side = 2, at= 1:P, las= 1, cex= 0.7, adj= 1.03)

#death cause-class association
cause_class_ass <- matrix(rep(0, C*P), ncol= P)
colnames(cause_class_ass)= sapply(colnames(data2), function(x) substr(x, 1, 13))
rownames(cause_class_ass)= seq(1, C)
treshold= .75 #quantile used as treshold for the inclusion of a death cause in a classù

for (i in 1:C){
  temp5= which(heatmapu_cprobs[i,] >= quantile(heatmapu_cprobs[i, ], treshold))
  temp6= which(heatmapu_diff[i, ] > 1 & heatmapu_diff[i, ] >= quantile(heatmapu_diff[i, ], treshold))
  temp7= intersect(temp5, temp6)
  cause_class_ass[i, temp7] <- 1 
}

temp8 = cause_class_ass[, order(colnames(cause_class_ass))]
image( 1:C, 1:P, temp8, col= c("white", "black"), axes = FALSE, xlab = "", ylab = "", 
       main= "Death cause - class association")
abline(v = 0:C+ 0.5, col= "gray")
abline(h = 0:P+ 0.5, col= "gray")
mtext(rownames(temp8), side = 1, at = 1:C, las = 1, cex = 0.7, adj = 0.5)
mtext(perc, side= 3, at= 1:C, las= 1, cex= .7, adj= .4)
mtext(colnames(temp8), side= 2, at= 1:P, las= 1, cex= 0.7, adj = 1.03)

dev.off()

